#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

#include "timer.h"
#include "driver.h"

#include "looptests.h"
#include "proctests.h"
#include "memtests.h"
#include "mysyscall.h"

//-----------------------------------------------------------------
// Subroutine to run memory tests.
//   We put this code in a subroutine because the thread interface
//   starts threads in a subroutine, and we run these tests
//   multithreaded.
//-----------------------------------------------------------------
void* runMemTests(void *ptid) {
  long int i;
  int loopCnt = ARRAYSIZE * 2;

  Timer timer = timer_new();

  TEMPLOCALITY(loopCnt);
  SPATIALLOCALITY(loopCnt);
  STRIDE32LOCALITY(loopCnt);
  STRIDE63LOCALITY(loopCnt);
  RANDOMLOCALITY(loopCnt, *(pthread_t*)ptid);

  timer_destroy(timer);

  return NULL;
}


//-----------------------------------
// main()
//-----------------------------------
int main()
{
  long int i;
  
  // All non-memory tests use this timer
  Timer timer = timer_new();
  if ( !timer ) {
    fprintf(stderr, "Couldn't create timer\n");
    exit(1);
  }

  //------------------------------------
  // Loop tests
  //------------------------------------
  printf("\nLoop tests:\n");
  
  int loopCnt = 8*20000000;

  TIME( NULLLOOP(loopCnt), loopCnt );
  TIME( LOOP(loopCnt), loopCnt );

  //------------------------------------
  // Procedure call tests
  //------------------------------------
  printf("\nProcedure call tests:\n");
  
  loopCnt = 20000000;

  NULLPROC(loopCnt);
  ONEPROC(loopCnt);
  EIGHTPROC(loopCnt);

  //-------------------------------------
  // Syscall overhead
  //-------------------------------------
  printf("\nSyscall tests:\n");

  loopCnt = 1000;
  SYSCALL_PID(loopCnt);
  SYSCALL_OPEN_CLOSE(loopCnt);
  SYSCALL_FOPEN_FCLOSE(loopCnt);
  SYSCALL_FORK_WAIT(loopCnt);
  THREAD_CREATE_JOIN(loopCnt);
  
  //------------------------------------
  // Memory tests: single-threaded
  //------------------------------------
  printf("\nSingle-threaded memory tests:\n");
  pthread_t my_tid = pthread_self();
  runMemTests(&my_tid);
  
  //------------------------------------
  // Memory tests: two-threads
  //   (The current thread is one of them.)
  //------------------------------------
  printf("\nTwo-threaded memory tests:\n");

  pthread_t helpers[7];
  pthread_create( &helpers[0], NULL, runMemTests, &helpers[0]);
  runMemTests(&my_tid);
  pthread_join(helpers[0], NULL);
  
  //------------------------------------
  // Memory tests: four threads
  //------------------------------------
  printf("\nFour-threaded memory tests:\n");

  for (int t=0; t<3; t++) 
    pthread_create( &helpers[t], NULL, runMemTests, &helpers[t]);
  runMemTests(&my_tid);
  for (int t=0; t<3; t++) 
    pthread_join(helpers[t], NULL);

  //------------------------------------
  // Memory tests: eight threads
  //------------------------------------
  printf("\nEight-threaded memory tests:\n");

  for (int t=0; t<7; t++) 
    pthread_create( &helpers[t], NULL, runMemTests, &helpers[t]);
  runMemTests(&my_tid);
  for (int t=0; t<7; t++) 
    pthread_join(helpers[t], NULL);

  // clean up
  timer_destroy(timer);
  
  return 0;
}

